/* upstart
 *
 * tty.c - tty change events
 *
 * Copyright © 2007 Canonical Ltd.
 * Author: Scott James Remnant <scott@ubuntu.com>.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif /* HAVE_CONFIG_H */

#include <unistd.h>
#include <stdlib.h>
#include <termios.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <nih/logging.h>
#include <nih/string.h>
#include <nih/alloc.h>

#include "paths.h"

static char **tty_stack = NULL;
static size_t tty_stack_top = 0;

/**
 * tty_stack_init:
 *
 * Allocate the tty stack
 **/
void
tty_stack_init (void)
{
	NIH_MUST (tty_stack = nih_str_array_new (NULL));
}

/**
 * current_tty:
 *
 * Get the tty at the top of the stack
 **/
const char *
current_tty (void)
{
	if (!tty_stack_top) return CONSOLE;
	return tty_stack[tty_stack_top - 1];
}

/**
 * tty_stack_init:
 *
 * Push a tty onto the tty stack
 **/
const char *
push_tty (const char *tty)
{
	nih_assert (tty_stack != NULL);
	NIH_MUST (nih_str_array_add (&tty_stack, NULL, &tty_stack_top, tty));

	return current_tty ();
}

/**
 * pop_tty:
 *
 * Pop a tty off of the tty stack
 **/
const char *
pop_tty (void)
{
	nih_assert (tty_stack != NULL);
	if (tty_stack_top == 0) return current_tty();
	nih_free (tty_stack[tty_stack_top - 1]);
	tty_stack[--tty_stack_top] = NULL;
	tty_stack = nih_realloc (tty_stack, NULL,
			sizeof (char *) * (tty_stack_top + 2));

	return current_tty ();
}

